@@ -172,9 +172,8 @@ class RouteEntry(lg.LookingGlassMixin):
         for attribute in self.attributes.values():
 
             # skip some attributes that we care less about
-            if (attribute.ID == exa.Attribute.CODE.AS_PATH or
-                    attribute.ID == exa.Attribute.CODE.ORIGIN or
-                    attribute.ID == exa.Attribute.CODE.LOCAL_PREF):
+            if attribute.ID in (exa.Attribute.CODE.AS_PATH,
+                                exa.Attribute.CODE.ORIGIN):
                 continue
 
             att_dict[
@@ -258,11 +257,18 @@ class RouteEvent(object):
                                                self.source)
 
 
+class UnsupportedRT(Exception):
+
+    def __init__(self, rt):
+        self.rt = rt
+
+
 class _SubUnsubCommon(object):
 
     def __init__(self, afi, safi, route_target, worker=None):
-        assert route_target is None or isinstance(route_target,
-                                                  exa.RouteTarget)
+        if not (route_target is None or
+                isinstance(route_target, exa.RouteTarget)):
+            raise UnsupportedRT(route_target)
         self.afi = afi
         self.safi = safi
         self.route_target = route_target
