@@ -18,7 +18,6 @@
 import collections
 from functools import reduce
 import threading
-import traceback
 
 from oslo_log import log as logging
 import six
@@ -26,6 +25,7 @@ from six.moves import queue
 
 from networking_bagpipe.bagpipe_bgp.common import log_decorator
 from networking_bagpipe.bagpipe_bgp.common import looking_glass as lg
+from networking_bagpipe.bagpipe_bgp.common import utils
 from networking_bagpipe.bagpipe_bgp import engine
 from networking_bagpipe.bagpipe_bgp.engine import bgp_peer_worker
 from networking_bagpipe.bagpipe_bgp.engine import exa
@@ -151,7 +151,8 @@ def test_should_dispatch(route_event, target_worker):
         return (True, "")
 
 
-class RouteTableManager(threading.Thread, lg.LookingGlassMixin):
+class RouteTableManager(threading.Thread, lg.LookingGlassMixin,
+                        utils.ClassReprMixin):
     """Singleton class dispatching route events between workers
 
     Events relates to BGP routes that are announced or withdrawn by workers.
@@ -214,10 +215,8 @@ class RouteTableManager(threading.Thread, lg.LookingGlassMixin):
                 self._worker_cleanup(event.worker)
             else:
                 raise Exception("unknown event: %s", event)
-        except Exception as e:
-            LOG.error("Exception during processing of event: %s", repr(e))
-            LOG.error("%s", traceback.format_exc())
-            LOG.error("    event was: %s", event)
+        except Exception:
+            LOG.exception("Exception during processing of event %s", event)
 
     def enqueue(self, event):
         self._queue.put(event)
@@ -416,8 +415,8 @@ class RouteTableManager(threading.Thread, lg.LookingGlassMixin):
 
             if replaced_entry == route_event.route_entry:
                 LOG.warning("Ignoring, the route advertized is the same as the"
-                            " one previously advertized by the source (%s)",
-                            route_event.source)
+                            " one previously advertized by the source %s: %s",
+                            route_event.source, route_event.route_entry)
                 return
 
             # propagate event to interested worker
