@@ -31,6 +31,8 @@ from networking_bagpipe.bagpipe_bgp.engine import ipvpn as ipvpn_routes
 from networking_bagpipe.bagpipe_bgp.vpn import dataplane_drivers as dp_drivers
 from networking_bagpipe.bagpipe_bgp.vpn import vpn_instance
 
+DEFAULT_ADDR_PREFIX = '0.0.0.0/0'
+
 
 @six.add_metaclass(abc.ABCMeta)
 class VPNInstanceDataplane(dp_drivers.VPNInstanceDataplane):
@@ -76,6 +78,18 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
         vpn_instance.VPNInstance.__init__(self, *args, **kwargs)
         self.readvertised = set()
 
+    def needs_cleanup_assist(self, afi, safi):
+        if (afi, safi) == (self.afi, exa.SAFI.flow_vpn):
+            return True
+        else:
+            return self.dataplane.needs_cleanup_assist()
+
+    @classmethod
+    def validate_convert_params(cls, params, also_mandatory=()):
+        return super(VRF, cls).validate_convert_params(
+            params,
+            also_mandatory=tuple(set(also_mandatory) | set(['ip_address'])))
+
     @classmethod
     def validate_convert_attach_params(cls, params):
         super(VRF, cls).validate_convert_attach_params(params)
@@ -99,15 +113,20 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
         for port_data in self.mac_2_localport_data.values():
             yield port_data['label']
 
+    def _get_route_params_for_endpoint(self, endpoint):
+        port_data = self.mac_2_localport_data[endpoint[0]]
+        label = port_data['label']
+        lb_consistent_hash_order = port_data[
+            'lb_consistent_hash_order']
+        rd = self.endpoint_2_rd[endpoint]
+
+        return (label, rd, lb_consistent_hash_order)
+
     def _imported(self, route):
         return len(set(route.route_targets).intersection(set(self.import_rts))
                    ) > 0
 
     def _to_readvertise(self, route):
-        # Only re-advertise IP VPN routes (e.g. not Flowspec routes)
-        if not isinstance(route.nlri, ipvpn_routes.IPVPN):
-            return False
-
         rt_records = route.ecoms(exa.RTRecord)
         self.log.debug("RTRecords: %s (readvertise_to_rts:%s)",
                        rt_records,
@@ -131,25 +150,29 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
             set(self.readvertise_from_rts)
             )) > 0
 
-    def _route_for_readvertisement(self, route, label, rd,
-                                   lb_consistent_hash_order,
-                                   do_default=False):
-        prefix = "0.0.0.0/0" if do_default else route.nlri.cidr.prefix()
+    def _gen_rtrecords_extended_community(self, ecoms):
+        # new RTRecord = original RTRecord (if any) + orig RTs converted into
+        # RTRecord attributes
+        orig_rtrecords = ecoms(exa.RTRecord)
+        rts = ecoms(exa.RTExtCom)
+        add_rtrecords = [exa.RTRecord.from_rt(rt) for rt in rts]
 
-        nlri = self._nlri_from(prefix, label, rd)
+        return list(set(orig_rtrecords) | set(add_rtrecords))
+
+    def _route_for_readvertisement(self, route, endpoint):
+        label, rd, lb_consistent_hash_order = (
+            self._get_route_params_for_endpoint(endpoint)
+        )
+        self.log.debug("Prefix %s (re-)advertisement with label %s and route "
+                       "distinguisher %s", route.nlri.cidr.prefix(), label, rd)
+        nlri = self._nlri_from(route.nlri.cidr.prefix(), label, rd)
 
         attributes = exa.Attributes()
 
-        # new RTRecord = original RTRecord (if any) + orig RTs converted into
-        # RTRecord attributes
-        orig_rtrecords = route.ecoms(exa.RTRecord)
-        rts = route.ecoms(exa.RTExtCom)
-        add_rtrecords = [exa.RTRecord.from_rt(rt) for rt in rts]
-
-        final_rtrecords = list(set(orig_rtrecords) | set(add_rtrecords))
-
         ecoms = self._gen_encap_extended_communities()
-        ecoms.communities += final_rtrecords
+        ecoms.communities += (
+            self._gen_rtrecords_extended_community(route.ecoms)
+        )
         ecoms.communities.append(
             exa.ConsistentHashSortOrder(lb_consistent_hash_order))
         attributes.add(ecoms)
@@ -158,11 +181,33 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
         self.log.debug("RouteEntry for (re-)advertisement: %s", entry)
         return entry
 
+    def _default_route_for_advertisement(self, endpoint):
+        label, rd, lb_consistent_hash_order = (
+            self._get_route_params_for_endpoint(endpoint)
+        )
+        self.log.debug("Default route (re-)advertisement with label %s and "
+                       "route distinguisher %s", label, rd)
+        nlri = self._nlri_from(DEFAULT_ADDR_PREFIX, label, rd)
+
+        attributes = exa.Attributes()
+
+        ecoms = self._gen_encap_extended_communities()
+        ecoms.communities.append(
+            exa.ConsistentHashSortOrder(lb_consistent_hash_order))
+        attributes.add(ecoms)
+
+        entry = engine.RouteEntry(nlri, self.readvertise_to_rts, attributes)
+        self.log.debug("RouteEntry for default prefix advertisement: %s",
+                       entry)
+        return entry
+
     @log_decorator.log
-    def _route_for_attract_static_dest_prefixes(self, label, rd):
+    def _routes_for_attract_static_dest_prefixes(self, endpoint):
         if not self.attract_static_dest_prefixes:
             return
 
+        label, rd, _ = self._get_route_params_for_endpoint(endpoint)
+
         for prefix in self.attract_static_dest_prefixes:
             nlri = self._nlri_from(prefix, label, rd)
 
@@ -185,75 +230,101 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
 
         return self.synthesize_redirect_bgp_route(rules)
 
-    def _advertise_route_or_default(self, route, label, rd,
-                                    lb_consistent_hash_order=0):
-        if self.attract_traffic:
-            self.log.debug("Advertising default route from VRF %d to "
-                           "redirection VRF", self.instance_id)
+    def _redirect_route_for_readvertisement(self, route):
+        # Create a FlowSpec NLRI with distinct RD and a copy of rules from
+        # FlowSpec route to readvertise
+        nlri = flowspec.FlowRouteFactory(self.afi, self.instance_rd)
+        nlri.rules = route.nlri.rules
 
-        route_entry = self._route_for_readvertisement(
-            route, label, rd, lb_consistent_hash_order,
-            do_default=self.attract_traffic
+        attributes = exa.Attributes()
+
+        ecoms = exa.ExtendedCommunities()
+        ecoms.communities += (
+            self._gen_rtrecords_extended_community(route.ecoms)
         )
-        self._advertise_route(route_entry)
-
-    def _withdraw_route_or_default(self, route, label, rd,
-                                   lb_consistent_hash_order=0):
-        if self.attract_traffic:
-            self.log.debug("Stop advertising default route from VRF to "
-                           "redirection VRF")
-
-        route_entry = self._route_for_readvertisement(
-            route, label, rd, lb_consistent_hash_order,
-            do_default=self.attract_traffic
+        assert len(self.attract_rts) == 1
+        rt = self.attract_rts[0]
+        ecoms.communities.append(
+            exa.TrafficRedirect(exa.ASN(int(rt.asn)), int(rt.number))
         )
-        self._withdraw_route(route_entry)
+        attributes.add(ecoms)
+
+        entry = engine.RouteEntry(nlri, self.readvertise_to_rts, attributes)
+        self.log.debug("RouteEntry for redirect (re-)advertisement: %s", entry)
+        return entry
 
     @log_decorator.log
     def _readvertise(self, route):
         nlri = route.nlri
 
-        self.log.debug("Start re-advertising %s from VRF", nlri.cidr.prefix())
-        for _, endpoints in self.localport_2_endpoints.items():
-            for endpoint in endpoints:
-                port_data = self.mac_2_localport_data[endpoint[0]]
-                label = port_data['label']
-                lb_consistent_hash_order = port_data[
-                    'lb_consistent_hash_order']
-                rd = self.endpoint_2_rd[endpoint]
-                self.log.debug("Start re-advertising %s from VRF, with label "
-                               "%s and route distinguisher %s",
-                               nlri, label, rd)
-                # need a distinct RD for each route...
-                self._advertise_route_or_default(route, label, rd,
-                                                 lb_consistent_hash_order)
-
+        self.log.debug("Start re-advertising %s from VRF", nlri)
         if self.attract_traffic:
-            flow_route = self._route_for_redirect_prefix(nlri.cidr.prefix())
-            self._advertise_route(flow_route)
+            # Start advertising default route only when the first route to
+            # readvertise appears
+            if not self.readvertised:
+                for endpoint in self.all_endpoints():
+                    self.log.debug("Start advertising default route from VRF "
+                                   "%d to redirection VRF", self.instance_id)
+                    self._advertise_route(
+                        self._default_route_for_advertisement(endpoint)
+                    )
+
+            if isinstance(nlri, flowspec.Flow):
+                # Readvertise FlowSpec route
+                self._advertise_route(
+                    self._redirect_route_for_readvertisement(route)
+                )
+            else:
+                # Advertise FlowSpec route for prefix
+                self._advertise_route(
+                    self._route_for_redirect_prefix(nlri.cidr.prefix())
+                )
+
+        else:
+            for endpoint in self.all_endpoints():
+                self.log.debug("Start re-advertising %s from endpoint %s",
+                               nlri.cidr.prefix(), endpoint)
+                self._advertise_route(
+                    self._route_for_readvertisement(route, endpoint)
+                )
 
         self.readvertised.add(route)
 
     @log_decorator.log
-    def _readvertise_stop(self, route):
+    def _readvertise_stop(self, route, last):
         nlri = route.nlri
 
-        self.log.debug("Stop re-advertising %s from VRF", nlri.cidr.prefix())
-        for _, endpoints in self.localport_2_endpoints.items():
-            for endpoint in endpoints:
-                port_data = self.mac_2_localport_data[endpoint[0]]
-                label = port_data['label']
-                lb_consistent_hash_order = port_data[
-                    'lb_consistent_hash_order']
-                rd = self.endpoint_2_rd[endpoint]
-                self.log.debug("Stop re-advertising %s from VRF, with label %s"
-                               "and route distinguisher %s", nlri, label, rd)
-                self._withdraw_route_or_default(route, label, rd,
-                                                lb_consistent_hash_order)
+        if last:
+            self.log.debug("Stop re-advertising %s from VRF", nlri)
+            if self.attract_traffic:
+                # Stop advertising default route only if the withdrawn route is
+                # the last of the routes to readvertise
+                if len(self.readvertised) == 1:
+                    for endpoint in self.all_endpoints():
+                        self.log.debug("Stop advertising default route from "
+                                       "VRF to redirection VRF")
+                        self._withdraw_route(
+                            self._default_route_for_advertisement(endpoint)
+                        )
 
-        if self.attract_traffic:
-            flow_route = self._route_for_redirect_prefix(nlri.cidr.prefix())
-            self._withdraw_route(flow_route)
+                if isinstance(nlri, flowspec.Flow):
+                    # Withdraw readvertised FlowSpec route
+                    self._withdraw_route(
+                        self._redirect_route_for_readvertisement(route)
+                    )
+                else:
+                    # Withdraw FlowSpec route for prefix
+                    self._withdraw_route(
+                        self._route_for_redirect_prefix(nlri.cidr.prefix())
+                    )
+            else:
+                for endpoint in self.all_endpoints():
+                    self.log.debug("Stop re-advertising %s from endpoint %s",
+                                   nlri.cidr.prefix(), endpoint)
+                    route_entry = (
+                        self._route_for_readvertisement(route, endpoint)
+                    )
+                    self._withdraw_route(route_entry)
 
         self.readvertised.remove(route)
 
@@ -268,41 +339,45 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
 
         if vpn_instance.forward_to_port(kwargs.get('direction')):
             endpoint = (mac_address, ip_address_prefix)
-            label = self.mac_2_localport_data[mac_address]['label']
-            rd = self.endpoint_2_rd[endpoint]
             for route in itertools.chain(
                     self.readvertised,
-                    self._route_for_attract_static_dest_prefixes(label, rd)):
+                    self._routes_for_attract_static_dest_prefixes(endpoint)):
                 self.log.debug("Re-advertising %s with this port as next hop",
                                route.nlri)
-                self._advertise_route_or_default(route, label, rd,
-                                                 lb_consistent_hash_order)
-
                 if self.attract_traffic:
-                    flow_route = self._route_for_redirect_prefix(
-                        route.nlri.cidr.prefix())
-                    self._advertise_route(flow_route)
+                    self._advertise_route(
+                        self._default_route_for_advertisement(endpoint)
+                    )
+
+                    if self.has_only_one_endpoint():
+                        self._advertise_route(self._route_for_redirect_prefix(
+                            route.nlri.cidr.prefix()))
+                else:
+                    self._advertise_route(
+                        self._route_for_readvertisement(route, endpoint)
+                    )
 
     @log_decorator.log_info
     def vif_unplugged(self, mac_address, ip_address_prefix):
         endpoint = (mac_address, ip_address_prefix)
         direction = self.endpoint_2_direction[endpoint]
         if vpn_instance.forward_to_port(direction):
-            label = self.mac_2_localport_data[mac_address]['label']
-            lb_consistent_hash_order = (self.mac_2_localport_data[mac_address]
-                                        ["lb_consistent_hash_order"])
-            rd = self.endpoint_2_rd[endpoint]
             for route in itertools.chain(
                     self.readvertised,
-                    self._route_for_attract_static_dest_prefixes(label, rd)):
+                    self._routes_for_attract_static_dest_prefixes(endpoint)):
                 self.log.debug("Stop re-advertising %s", route.nlri)
-                self._withdraw_route_or_default(route, label, rd,
-                                                lb_consistent_hash_order)
+                if self.attract_traffic:
+                    self._withdraw_route(
+                        self._default_route_for_advertisement(endpoint)
+                    )
 
-                if self.attract_traffic and self.has_only_one_endpoint():
-                    flow_route = self._route_for_redirect_prefix(
-                        route.nlri.cidr.prefix())
-                    self._withdraw_route(flow_route)
+                    if self.has_only_one_endpoint():
+                        self._withdraw_route(self._route_for_redirect_prefix(
+                            route.nlri.cidr.prefix()))
+                else:
+                    self._withdraw_route(
+                        self._route_for_readvertisement(route, endpoint)
+                    )
 
         super(VRF, self).vif_unplugged(mac_address, ip_address_prefix)
 
@@ -322,33 +397,33 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
     @log_decorator.log
     def new_best_route(self, entry, new_route):
 
+        if self.readvertise:
+            # check if this is a route we need to re-advertise
+            self.log.debug("route RTs: %s", new_route.route_targets)
+            self.log.debug("readv from RTs: %s", self.readvertise_from_rts)
+            if self._to_readvertise(new_route):
+                self.log.debug("Need to re-advertise %s", entry)
+                self._readvertise(new_route)
+
+        if not self._imported(new_route):
+            self.log.debug("No need to setup dataplane for:%s", entry)
+            return
+
         if isinstance(new_route.nlri, flowspec.Flow):
             if len(new_route.ecoms(exa.TrafficRedirect)) == 1:
                 traffic_redirect = new_route.ecoms(exa.TrafficRedirect)
                 redirect_rt = "%s:%s" % (traffic_redirect[0].asn,
                                          traffic_redirect[0].target)
 
-                self.start_redirect_traffic(redirect_rt, new_route.nlri.rules)
+                self.start_redirect_traffic(redirect_rt,
+                                            new_route.nlri.rules)
             else:
-                self.log.warning("FlowSpec action or multiple traffic redirect"
-                                 " actions not supported: %s",
+                self.log.warning("FlowSpec action or multiple traffic "
+                                 "redirect actions not supported: %s",
                                  new_route.ecoms())
         else:
             prefix = entry
 
-            if self.readvertise:
-                # check if this is a route we need to re-advertise
-                self.log.debug("route RTs: %s", new_route.route_targets)
-                self.log.debug("readv from RTs: %s", self.readvertise_from_rts)
-                if self._to_readvertise(new_route):
-                    self.log.debug("Need to re-advertise %s", prefix)
-                    self._readvertise(new_route)
-
-            if not self._imported(new_route):
-                self.log.debug("No need to setup dataplane for:%s",
-                               prefix)
-                return
-
             encaps = self._check_encaps(new_route)
             if not encaps:
                 return
@@ -369,29 +444,30 @@ class VRF(vpn_instance.VPNInstance, lg.LookingGlassMixin):
     @log_decorator.log
     def best_route_removed(self, entry, old_route, last):
 
-        if isinstance(old_route.nlri, flowspec.Flow):
-            if len(old_route.ecoms(exa.TrafficRedirect)) == 1:
-                if last:
-                    traffic_redirect = old_route.ecoms(
-                        exa.TrafficRedirect)
-                    redirect_rt = "%s:%s" % (traffic_redirect[0].asn,
-                                             traffic_redirect[0].target)
+        if self.readvertise:
+            # check if this is a route we were re-advertising
+            if self._to_readvertise(old_route):
+                self.log.debug("Need to stop re-advertising %s", entry)
+                self._readvertise_stop(old_route, last)
 
-                    self.stop_redirect_traffic(redirect_rt,
-                                               old_route.nlri.rules)
-            else:
-                self.log.warning("FlowSpec action or multiple traffic redirect"
-                                 " actions not supported: %s",
-                                 old_route.ecoms())
+        if isinstance(old_route.nlri, flowspec.Flow):
+            if self._imported(old_route):
+                if len(old_route.ecoms(exa.TrafficRedirect)) == 1:
+                    if last:
+                        traffic_redirect = old_route.ecoms(
+                            exa.TrafficRedirect)
+                        redirect_rt = "%s:%s" % (traffic_redirect[0].asn,
+                                                 traffic_redirect[0].target)
+
+                        self.stop_redirect_traffic(redirect_rt,
+                                                   old_route.nlri.rules)
+                else:
+                    self.log.warning("FlowSpec action or multiple traffic "
+                                     "redirect actions not supported: %s",
+                                     old_route.ecoms())
         else:
             prefix = entry
 
-            if self.readvertise and last:
-                # check if this is a route we were re-advertising
-                if self._to_readvertise(old_route):
-                    self.log.debug("Need to stop re-advertising %s", prefix)
-                    self._readvertise_stop(old_route)
-
             # NOTE(tmorin): On new best routes, we only trigger dataplane
             # update events after checking with self._imported(...) that the
             # route was imported (and not a route that we receive because the
