@@ -19,8 +19,6 @@ import random
 
 from networking_bagpipe.tests.fullstack.resources.bagpipe_ml2 \
     import config as bagpipe_ml2_cfg
-from networking_bagpipe.tests.fullstack.resources.bagpipe_ml2 \
-    import process as bagpipe_ml2_proc
 from networking_bagpipe.tests.fullstack.resources.bgpvpn \
     import config as bgpvpn_cfg
 from networking_bagpipe.tests.fullstack.resources.common \
@@ -57,8 +55,8 @@ class BaGPipeEnvironmentDescription(neutron_env.EnvironmentDescription):
 
         self.bagpipe_ml2 = bagpipe_ml2
         self.bgpvpn = bgpvpn
-        self.evpn_driver = evpn_driver if bagpipe_ml2 else 'dummy'
-        self.ipvpn_driver = ipvpn_driver if bgpvpn else 'dummy'
+        self.evpn_driver = evpn_driver
+        self.ipvpn_driver = ipvpn_driver
         self.ipvpn_encap = ipvpn_encap
 
 
@@ -120,10 +118,7 @@ class BaGPipeHost(neutron_env.Host):
         )
         self.useFixture(agent_cfg_fixture)
 
-        if self.env_desc.bagpipe_ml2:
-            agent_fixture_cls = bagpipe_ml2_proc.LinuxBridgeAgentFixture
-        else:
-            agent_fixture_cls = neutron_proc.LinuxBridgeAgentFixture
+        agent_fixture_cls = neutron_proc.LinuxBridgeAgentFixture
 
         self.linuxbridge_agent = self.useFixture(
             agent_fixture_cls(
